@@ -1,4 +1,3 @@
-# $Id$
 package Test::File;
 use strict;
 
@@ -17,13 +16,15 @@ use Test::Builder;
 	file_is_symlink_ok
 	symlink_target_exists_ok symlink_target_is
 	symlink_target_dangles_ok
+	dir_exists_ok dir_contains_ok
 	link_count_is_ok link_count_gt_ok link_count_lt_ok
 	owner_is owner_isnt
 	group_is group_isnt
-    file_line_count_is file_line_count_isnt file_line_count_between
+	file_line_count_is file_line_count_isnt file_line_count_between
+	file_contains_like file_contains_unlike
 	);
 
-$VERSION = '1.28';
+$VERSION = '1.30';
 
 {
 use warnings;
@@ -54,6 +55,9 @@ tests automatically skip if they think they won't work on the
 platform.  If you have a way to make these functions work on Windows,
 for instance, please send me a patch. :)
 
+The optional NAME parameter for every function allows you to specify a name for the test.  If not
+supplied, a reasonable default will be generated.
+
 =head2 Functions
 
 =cut
@@ -74,7 +78,7 @@ sub _win32
 	return $^O =~ m/Win/;
 	}
 
-# returns true if symlinkscan't exist
+# returns true if symlinks can't exist
 sub _no_symlinks_here { ! eval { symlink("",""); 1 } }
 
 # owner_is and owner_isn't should skip on OS where the question makes no
@@ -86,6 +90,8 @@ sub _obviously_non_multi_user
 	{
 	foreach my $os ( qw(dos MacOS) ) { return 1 if $^O eq $os }
 
+	return 0 if $^O eq 'MSWin32';
+
 	eval { my $holder = getpwuid(0) };
 	return 1 if $@;
 
@@ -321,9 +327,9 @@ sub file_min_size_ok
 =item file_line_count_is( FILENAME, COUNT [, NAME ]  )
 
 Ok if the file exists and has COUNT lines (exactly), not ok if the
-file does not exist or exists with a line count than COUNT.
+file does not exist or exists with a line count other than COUNT.
 
-This function using the current value of C<$/> as the line ending and
+This function uses the current value of C<$/> as the line ending and
 counts the lines by reading them and counting how many it read.
 
 =cut
@@ -395,7 +401,7 @@ Ok if the file exists and doesn't have exactly COUNT lines, not ok if the
 file does not exist or exists with a line count of COUNT. Read that
 carefully: the file must exist for this test to pass!
 
-This function using the current value of C<$/> as the line ending and
+This function uses the current value of C<$/> as the line ending and
 counts the lines by reading them and counting how many it read.
 
 =cut
@@ -446,7 +452,7 @@ sub file_line_count_isnt
 
 Ok if the file exists and has a line count between MIN and MAX, inclusively.
 
-This function using the current value of C<$/> as the line ending and
+This function uses the current value of C<$/> as the line ending and
 counts the lines by reading them and counting how many it read.
 
 =cut
@@ -499,7 +505,133 @@ sub file_line_count_between
 		}
 
 	}
-	
+
+=item file_contains_like ( FILENAME, PATTERN [, NAME ] )
+
+Ok if the file exists and its contents (as one big string) match
+PATTERN, not ok if the file does not exist, is not readable, or exists
+but doesn't match PATTERN.
+
+Since the file contents are read into memory, you should not use this
+for large files.  Besides memory consumption, test diagnostics for
+failing tests might be difficult to decipher.  However, for short
+files this works very well.
+
+Because the entire contents are treated as one large string, you can
+make a pattern that tests multiple lines.  Don't forget that you may
+need to use the /s modifier for such patterns:
+
+	# make sure file has one or more paragraphs with CSS class X
+	file_contains_like($html_file, qr{<p class="X">.*?</p>}s);
+
+Contrariwise, if you need to match at the beginning or end of a line
+inside the file, use the /m modifier:
+
+	# make sure file has a setting for foo
+	file_contains_like($config_file, qr/^ foo \s* = \s* \w+ $/mx);
+
+If you want to test your file contents against multiple patterns, but
+don't want to have the file read in repeatedly, you can pass an
+arrayref of patterns instead of a single pattern, like so:
+
+	# make sure our template has rendered correctly
+	file_contains_like($template_out,
+		[
+		qr/^ $title_line $/mx,
+		map { qr/^ $_ $/mx } @chapter_headings,
+		qr/^ $footer_line $/mx,
+		]);
+
+Please note that if you do this, and your file does not exist or is
+not readable, you'll only get one test failure instead of a failure
+for each pattern.  This could cause your test plan to be off, although
+you may not care at that point because your test failed anyway.  If
+you do care, either skip the test plan altogether by employing
+L<Test::More>'s C<done_testing()> function, or use
+L</file_readable_ok> in conjunction with a C<SKIP> block.
+
+Contributed by Buddy Burden C<< <barefoot@cpan.org> >>.
+
+=item file_contains_unlike ( FILENAME, PATTERN [, NAME ] )
+
+Ok if the file exists and its contents (as one big string) do B<not>
+match PATTERN, not ok if the file does not exist, is not readable, or
+exists but matches PATTERN.
+
+All notes and caveats for L</file_contains_like> apply to this
+function as well.
+
+Contributed by Buddy Burden C<< <barefoot@cpan.org> >>.
+
+=cut
+
+sub file_contains_like
+	{
+		local $Test::Builder::Level = $Test::Builder::Level + 1;
+		_file_contains(like => "contains", @_);
+	}
+
+sub file_contains_unlike
+	{
+		local $Test::Builder::Level = $Test::Builder::Level + 1;
+		_file_contains(unlike => "doesn't contain", @_);
+	}
+
+sub _file_contains
+	{
+	my $method   = shift;
+	my $verb     = shift;
+	my $filename = _normalize( shift );
+	my $patterns = shift;
+	my $name     = shift;
+
+	my (@patterns, %patterns);
+	if (ref $patterns eq 'ARRAY')
+		{
+		@patterns = @$patterns;
+		%patterns = map { $_ => $name || "$filename $verb $_" } @patterns;
+		}
+		else
+		{
+		@patterns = ($patterns);
+		%patterns = ( $patterns => $name || "$filename $verb $patterns" );
+		}
+
+	# for purpose of checking the file's existence, just use the first
+	# test name as the name
+	$name = $patterns{$patterns[0]};
+
+	unless( -e $filename )
+		{
+		$Test->diag( "File [$filename] does not exist!" );
+		return $Test->ok(0, $name);
+		}
+
+	unless( -r $filename )
+		{
+		$Test->diag( "File [$filename] is not readable!" );
+		return $Test->ok(0, $name);
+		}
+
+	# do the slurp
+	my $file_contents;
+	{
+	unless (open(FH, $filename))
+		{
+		$Test->diag( "Could not open [$filename]: \$! is [$!]!" );
+		return $Test->ok( 0, $name );
+		}
+	local $/ = undef;
+	$file_contents = <FH>;
+	close FH;
+	}
+
+	foreach my $p (@patterns)
+		{
+		$Test->$method($file_contents, $p, $patterns{$p});
+		}
+	}
+
 =item file_readable_ok( FILENAME [, NAME ] )
 
 Ok if the file exists and is readable, not ok
@@ -744,14 +876,12 @@ sub file_mode_isnt
 		}
 	}
 
-=item file_is_symlink_ok( FILENAME [, NAME] )
+=item file_is_symlink_ok( FILENAME [, NAME ] )
 
-Ok is FILENAME is a symlink, even if it points to a non-existent
+Ok if FILENAME is a symlink, even if it points to a non-existent
 file. This test automatically skips if the operating system does
 not support symlinks. If the file does not exist, the test fails.
 
-The optional NAME parameter is the name of the test.
-
 =cut
 
 sub file_is_symlink_ok
@@ -777,15 +907,13 @@ sub file_is_symlink_ok
 		}
 	}
 
-=item symlink_target_exists_ok( SYMLINK [, TARGET] [, NAME] )
+=item symlink_target_exists_ok( SYMLINK [, TARGET] [, NAME ] )
 
-Ok is FILENAME is a symlink and it points to a existing file. With the
+Ok if FILENAME is a symlink and it points to a existing file. With the
 optional TARGET argument, the test fails if SYMLINK's target is not
 TARGET. This test automatically skips if the operating system does not
 support symlinks. If the file does not exist, the test fails.
 
-The optional NAME parameter is the name of the test.
-
 =cut
 
 sub symlink_target_exists_ok
@@ -828,14 +956,12 @@ sub symlink_target_exists_ok
 	$Test->ok( 1, $name );
 	}
 
-=item symlink_target_dangles_ok( SYMLINK [, NAME] )
+=item symlink_target_dangles_ok( SYMLINK [, NAME ] )
 
 Ok if FILENAME is a symlink and if it doesn't point to a existing
 file. This test automatically skips if the operating system does not
 support symlinks. If the file does not exist, the test fails.
 
-The optional NAME parameter is the name of the test.
-
 =cut
 
 sub symlink_target_dangles_ok
@@ -867,14 +993,12 @@ sub symlink_target_dangles_ok
 	$Test->ok( 1, $name );
 	}
 
-=item symlink_target_is( SYMLINK, TARGET [, NAME] )
+=item symlink_target_is( SYMLINK, TARGET [, NAME ] )
 
 Ok if FILENAME is a symlink and if points to TARGET. This test
 automatically skips if the operating system does not support symlinks.
 If the file does not exist, the test fails.
 
-The optional NAME parameter is the name of the test.
-
 =cut
 
 sub symlink_target_is 
@@ -918,14 +1042,12 @@ sub symlink_target_is
 		}
 	}
 
-=item symlink_target_is_absolute_ok( SYMLINK [, NAME] )
+=item symlink_target_is_absolute_ok( SYMLINK [, NAME ] )
 
 Ok if FILENAME is a symlink and if its target is an absolute path.
 This test automatically skips if the operating system does not support
 symlinks. If the file does not exist, the test fails.
 
-The optional NAME parameter is the name of the test.
-
 =cut
 
 =pod
@@ -964,16 +1086,79 @@ if (defined( $link_abs ) && defined( $to_abs ) && $link_abs eq $to_abs) {
 }
 }
 
+=item dir_exists_ok( DIRECTORYNAME [, NAME ] )
+
+Ok if the file exists and is a directory, not ok if the file doesn't exist, or exists but isn't a
+directory.
+
+Contributed by Buddy Burden C<< <barefoot@cpan.org> >>.
+
 =cut
 
-=item link_count_is_ok( FILE, LINK_COUNT [, NAME] )
+sub dir_exists_ok
+	{
+	my $filename = _normalize( shift );
+	my $name     = shift || "$filename is a directory";
+
+	unless( -e $filename )
+		{
+		$Test->diag( "File [$filename] does not exist!" );
+		return $Test->ok(0, $name);
+		}
+
+	my $ok = -d $filename;
+
+	if( $ok )
+		{
+		$Test->ok(1, $name);
+		}
+	else
+		{
+		$Test->diag( "File [$filename] exists but is not a directory!" );
+		$Test->ok(0, $name);
+		}
+	}
+
+=item dir_contains_ok( DIRECTORYNAME, FILENAME [, NAME ] )
+
+Ok if the directory exists and contains the file, not ok if the directory doesn't exist, or exists
+but doesn't contain the file.
+
+Contributed by Buddy Burden C<< <barefoot@cpan.org> >>.
+
+=cut
+
+sub dir_contains_ok
+	{
+	my $dirname  = _normalize( shift );
+	my $filename = _normalize( shift );
+	my $name     = shift || "directory $dirname contains file $filename";
+
+	unless( -d $dirname )
+		{
+		$Test->diag( "Directory [$dirname] does not exist!" );
+		return $Test->ok(0, $name);
+		}
+
+	my $ok = -e File::Spec->catfile($dirname, $filename);
+
+	if( $ok )
+		{
+		$Test->ok(1, $name);
+		}
+	else
+		{
+		$Test->diag( "File [$filename] does not exist in directory $dirname!" );
+		$Test->ok(0, $name);
+		}
+	}
+
+=item link_count_is_ok( FILE, LINK_COUNT [, NAME ] )
 
 Ok if the link count to FILE is LINK_COUNT. LINK_COUNT is interpreted
 as an integer. A LINK_COUNT that evaluates to 0 returns Ok if the file
 does not exist.
 
-The optional NAME parameter is the name of the test.
-
 
 =cut
 
@@ -996,14 +1181,12 @@ sub link_count_is_ok
 	$Test->ok( 1, $name );
 	}
 
-=item link_count_gt_ok( FILE, LINK_COUNT [, NAME] )
+=item link_count_gt_ok( FILE, LINK_COUNT [, NAME ] )
 
 Ok if the link count to FILE is greater than LINK_COUNT. LINK_COUNT is
 interpreted as an integer. A LINK_COUNT that evaluates to 0 returns Ok
 if the file has at least one link.
 
-The optional NAME parameter is the name of the test.
-
 =cut
 
 sub link_count_gt_ok
@@ -1026,14 +1209,12 @@ sub link_count_gt_ok
 	$Test->ok( 1, $name );
 	}
 
-=item link_count_lt_ok( FILE, LINK_COUNT [, NAME] )
+=item link_count_lt_ok( FILE, LINK_COUNT [, NAME ] )
 
 Ok if the link count to FILE is less than LINK_COUNT. LINK_COUNT is
 interpreted as an integer. A LINK_COUNT that evaluates to 0 returns Ok
 if the file has at least one link.
 
-The optional NAME parameter is the name of the test.
-
 =cut
 
 sub link_count_lt_ok
@@ -1090,14 +1271,12 @@ sub _dm_skeleton
 	return;
 	}
 
-=item owner_is( FILE , OWNER [, NAME] )
+=item owner_is( FILE , OWNER [, NAME ] )
 
 Ok if FILE's owner is the same as OWNER.  OWNER may be a text user name
 or a numeric userid.  Test skips on Dos, and Mac OS <= 9.
 If the file does not exist, the test fails.
 
-The optional NAME parameter is the name of the test.
-
 Contributed by Dylan Martin
 
 =cut
@@ -1141,14 +1320,12 @@ sub owner_is
 	return $Test->ok( 0, $name );
 	}
 
-=item owner_isnt( FILE, OWNER [, NAME] )
+=item owner_isnt( FILE, OWNER [, NAME ] )
 
 Ok if FILE's owner is not the same as OWNER.  OWNER may be a text user name
 or a numeric userid.  Test skips on Dos and Mac OS <= 9.  If the file
 does not exist, the test fails.
 
-The optional NAME parameter is the name of the test.
-
 Contributed by Dylan Martin
 
 =cut
@@ -1178,15 +1355,13 @@ sub owner_isnt
 	return $Test->ok( 0, $name );
 	}
 
-=item group_is( FILE , GROUP [, NAME] )
+=item group_is( FILE , GROUP [, NAME ] )
 
 Ok if FILE's group is the same as GROUP.  GROUP may be a text group name or
 a numeric group id.  Test skips on Dos, Mac OS <= 9 and any other operating
 systems that do not support getpwuid() and friends.  If the file does not
 exist, the test fails.
 
-The optional NAME parameter is the name of the test.
-
 Contributed by Dylan Martin
 
 =cut
@@ -1231,15 +1406,13 @@ sub group_is
 	return $Test->ok( 0, $name );
 	}
 
-=item group_isnt( FILE , GROUP [, NAME] )
+=item group_isnt( FILE , GROUP [, NAME ] )
 
 Ok if FILE's group is not the same as GROUP.  GROUP may be a text group name or
 a numeric group id.  Test skips on Dos, Mac OS <= 9 and any other operating
 systems that do not support getpwuid() and friends.  If the file does not
 exist, the test fails.
 
-The optional NAME parameter is the name of the test.
-
 Contributed by Dylan Martin
 
 =cut
@@ -1339,13 +1512,17 @@ some functions.
 
 Tom Metro helped me figure out some Windows capabilities.
 
-Dylan Martin added C<owner_is> and C<owner_isnt>
+Dylan Martin added C<owner_is> and C<owner_isnt>.
 
 David Wheeler added C<file_line_count_is>.
 
+Buddy Burden C<< <barefoot@cpan.org> >> provided C<dir_exists_ok>,
+C<dir_contains_ok>, C<file_contains_like>, and
+C<file_contains_unlike>.
+
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2002-2008 brian d foy.  All rights reserved.
+Copyright (c) 2002-2011 brian d foy.  All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
